
/* ====================================================
 * drawKmlData : reads kml file and plots trail on
 *               sun raster graphics map
 *
 * Core types of data handled
 *   points --------------------------------------------
 *      <Placemark>
 *          <Point>
 *               <coordinates>
 *                   lat,lon(,elev) ...
 *               </coordinates>
 *          </Point>
 *      </Placemark>
 *
 *   trails ----------------------------------------------
 *      <Placemark>
 *          <LineString>
 *               <coordinates>
 *                   lat,lon(,elev) ...
 *               </coordinates>
 *          </LineString>
 *      </Placemark>
 *
 *   boundaries ------------------------------------------
 *      <Placemark>
 *          <Polygon>(<outerBoundaryIs>|<innerBoundaryIs>)
 *                <LinearRing>
 *                    <coordinates>
 *                       lat,lon(,elec) ...
 *                    </coordinates>
 *                </LinearRing>
 *          (</outerBoundaryIs>|</innerBoundaryIs>)</Polygon>
 *
 * Other optional tags
 *    style with ID
 *       
 *    style without ID
 *       <Style>
 *          <LineStyle>
 *             <color>ff0000ff</color>
 *          </LineStyle>
 *       </Style>
 *
 *     discription with name as data
 *         <description><![CDATA[
 *           <b>NAME:</b> <i>(text)</i>
 *         ]]></description>
 *
 *     name as tag
 *        <name>(text}</name.
 8
 * =================================================== */

/* Standard includes
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

/* Application includes
 */
#include "drawmap.h"
#include "drawTrails.h"
#include "drawKmlData.h"

/* Internal Structures
 */
typedef struct {
  char styleID[24];
  char styleColor[12]; /* #oobbggrr\0 */
} LineStyleStruct;

/* global variables
 */
 FILE *kmlFile    = NULL;
 int   coordinate =    0;
 int   parseLevel = NULL_LVL;
 
 char  lastStyleID[80];           /* <Style id=      */
 char  lastLineStyle[24];         /*   <LineStyle>   */
 char  lastLineColor[12];         /*      <color>ffbbggrr</color> */
 LineStyleStruct lineStyle[20];      
 int   styleCnt   = 0;
 
 char *docBeginTag  = "<Document>";
 char *docEndTag    = "</Document>";
 char *foldrOnTag   = "<Folder>";
 char *foldrOffTag  = "</Folder>";
 char *placeOnTag   = "<Placemark>";
 char *placeOffTag  = "</Placemark>";
 char *coordOnTag   = "<coordinates>";
 char *coordOffTag  = "</coordinates>";
 char *styleIdTag   = "<Style id=";
 char *lineStyleTag = "<LineStyle>";
 char *colorOnTag   = "<color>";
 char *colorOffTag  = "</color>";
 char *styleEndTag  = "</Style>";
 char *nameOnTag    = "<name>";
 char *nameOffTag   = "</name>";
 char *pointOnTag   = "<Point>";
 char *pointOffTag  = "</Point>";
 char *lineOnTag    = "<LineString>";
 char *lineOffTag   = "</LineString>";
 char *PolyOnTag    = "<Polygon>";
 char *PolyOffTag   = "</Polygon>";

int openKmlFile(char *fileName)
{
  if ((kmlFile = fopen(fileName, "r")) != NULL) {
    parseLevel = NULL_LVL;
    return 1;
  }
  else
    return 0;
}

int matchTag(char *line, char *tag)
{
  int   tagLen = strlen(tag);
  char *start  = line;

  /* Step over leading white space
   */
  while ((*start == ' ') || (*start == '\t')) start += 1;

  if (strncmp(start, tag, tagLen) == 0)
    return 1;
  else
    return 0;
}

char *copyQuoted(char *line, char leftQuote, char rightQuote)
{
  static char buffer[80];
  char       *left = index (line, leftQuote );
  char       *right= rindex(line, rightQuote);
  long        len;

  memset(buffer, 0, 80);
  if ((left) && (right)) {
    left += 1;
    len  = (long)(right - left);
    strncpy(buffer, left, len);
  }
  return buffer;
}

/* updateLevel : watches for the end of the header data
 */
int updateLevel(int prev_lvl, char *line)
{
  static int folderPending = 0;

  if (matchTag(line, docBeginTag)) {
    prev_lvl = DOCMT_LVL;
  }
  else if (matchTag(line, docEndTag  )) {
    prev_lvl = NULL_LVL;
  }
  else if (matchTag(line, foldrOnTag )) {
    prev_lvl = FOLDR_LVL;
  }
  else if (matchTag(line, styleIdTag)) {
    strcpy(lastStyleID, copyQuoted(line, '"', '"')); /* save the style id,      */
    memset(lastLineStyle,   0, 20);                 /* may not be a line style */
    memset(lastLineColor,   0, 12);                 /* clear color             */
    prev_lvl = STYLE_LVL;
  }
  else if (matchTag(line, placeOnTag)) {
    prev_lvl = PLACE_LVL;
  }
  
  return prev_lvl;
}

/* ******************************************************
 * int getPhrase
 *       input  character from input stream
 *       output complete KML phrase
 *       in/out parslvl
 */
#define PHRASE_NEW        0
#define PHRASE_BUILD_TAG  1
#define PHRASE_BUILD_DATA 2
#define PHRASE_DONE       3
#define PHRASE_OVER_RUN  -1

char *getPhrase(char *line, int lineLen, FILE *kmlFile)
{
  static int state  = PHRASE_NEW;
  int  cntr         = 0;
  char nextChar;
  int  waitForQuote = 0;
  int  stopLen      = lineLen-2;

  memset(line, 0, lineLen);
  if (state == PHRASE_DONE) {        /* Restart if last was success */  
    state = PHRASE_NEW;
  }
  while ((nextChar = fgetc(kmlFile)) != EOF) {
    switch (state) {
      case PHRASE_OVER_RUN:                            /* data was too long, flush    */
        if ((nextChar =='\n'  ) || 
	    (isblank(nextChar)) ||                     /* found white spaces          */
	    (nextChar == '>'  )   ) {                  /* end of old tag              */
	  state = PHRASE_NEW;
	} else if (nextChar =='<')  {                  /* start of new tag            */
	    line[cntr++] = nextChar;
	    state = PHRASE_BUILD_TAG;
	}
	break;

      case PHRASE_NEW:                                 /* get start of new phrase     */
        if ((nextChar =='\n') || (isblank(nextChar))) { /* ignore white spaces         */
	  break;
        }
	else {
	  line[cntr++] = nextChar;
	  if (nextChar == '<') {                       /* we are starting a tag       */
	    state = PHRASE_BUILD_TAG;
	  }
	  else {
	    state = PHRASE_BUILD_DATA;                 /* we are starting data        */
	    if (nextChar == '"') {                     /* Allow spaces in quoted data */
	      waitForQuote = 1 ;
	    }
	  }
	}
	break;

      case PHRASE_BUILD_TAG:
	line[cntr++] = nextChar;
	if (nextChar == '>') {
	  state = PHRASE_DONE;
	  return line;
        }
	break;
			     
      case PHRASE_BUILD_DATA:
	if (nextChar == '"') {
	  waitForQuote = (!waitForQuote);
	}

	if (!waitForQuote) {
	  if ((nextChar == '<' )  || 
              (nextChar == '\n')  || 
              (isblank(nextChar))   ) {
	    state = PHRASE_DONE;
	    if (nextChar == '<') {
	      ungetc(nextChar, kmlFile);             /* Put it back for next read */
            }
	    return line;
	  }
	}
        line[cntr++] = nextChar;
	break;
    }
    if (cntr > stopLen)
      return line;
  }

  return NULL;
}

int parseKmlStyle(int prev_lvl, char *line)
{
  static int colorData = 0;

  if (matchTag(line, lineStyleTag)) {
      strcpy(lastLineStyle, lastStyleID);
      colorData = 0;
  }
  else if (matchTag(line, colorOnTag)) {
      colorData = 1;
  }
  else if (matchTag(line, colorOffTag)) {
      colorData = 0;
  }
  else if (matchTag(line, styleEndTag)) {
    if (strlen(lastLineStyle) > 1) {
      colorData = 0;
      strncpy(lineStyle[styleCnt].styleID,    lastLineStyle, 23);
      strncpy(lineStyle[styleCnt].styleColor, lastLineColor, 11);     
      styleCnt++;
      fprintf(stderr, "Debug: style %s color %s\n", lastLineStyle, lastLineColor);
    }
    memset(lastStyleID, 0, 80);
    prev_lvl = DOCMT_LVL;
  }
  else if (colorData) {
    strcpy(lastLineColor, line);
  }

  return prev_lvl;
}

int parseFolderName(int prev_lvl, char *line)
{
  static int nameData = 0;

  if (matchTag(line, nameOnTag)) {
    nameData = 1;
  }
  else if (matchTag(line, nameOffTag)) {
    nameData = 0;
  }
  else if (nameData) {
    char name[80];
   
    strncpy(name, line, 79);
    if (strcmp(name, "Tracks") == 0) {
      prev_lvl = TRACK_LVL;
    }
    nameData = 0;
  }
  return prev_lvl;
}

int parseKmlHeader()
{
  char line[255];
  char name[ 80];
  int  colordata = 0;

  while ((getPhrase(line, 254, kmlFile)) && (parseLevel < TRACK_LVL)) {
    /* If parseLevel is active, process any data first
     */
    switch (parseLevel) {
      case STYLE_LVL:
	              parseLevel = parseKmlStyle(parseLevel, line);
		      break;
      case FOLDR_LVL:
	/* parseLevel = parseFolderName(parseLevel, line); */
		      break;
      case NAME_LVL: 
	              strncpy(name, line, 80);    
		      if (strcmp(name, "Tracks") == 0) {
			parseLevel = TRACK_LVL;
		      }
		      break;

      case DOCMT_LVL:
      case TRACK_LVL:
      default:
	              break;
    }
    /* Test for new parse level.
     */
    parseLevel = updateLevel(parseLevel, line); 
  }
}

/* **********************************************************
 * nextKmlCoordinate 
 */


int nextKmlCoordinate(int *mode, double *latitude, double *longitude, int *restart)
{
  char   line[255];
  double elev;
  

  if (kmlFile == NULL) {
    return 0;
  }

  while (getPhrase(line, 254, kmlFile)) {
    /* look for object type     */
    if (strstr(line, pointOnTag)) {
      *mode = POINT_COORD;
    }
    else if (strstr(line, lineOnTag)) {
      *mode = LINE_COORD;
    } 
    else if (strstr(line, PolyOnTag)) {
      *mode = POLY_COORD;
    }

    /* look for style Tag */
    if (strstr(line, "lineStyle")!= NULL)  {
      char colorStyle[80],
           *color;
      strcpy(colorStyle, line+1);
      color = colorStyle+9;
      SetPlotColor(color);

    }

    /* look for coordinate Tag */
    if (strstr(line, coordOnTag) != NULL) {
      *restart   = 1;
      coordinate = 1;
    }
    else if (strstr(line, coordOffTag)!= NULL) {
      coordinate = 0;
    }
    else if (coordinate) {
      if (sscanf(line, " %lf,%lf,%lf", longitude, latitude, &elev) >= 2) {
	  return 1;
      }
    }
  }
  return 0;
}

void closeKmlFile()
{
  if (kmlFile != NULL) {
    fclose(kmlFile);
    kmlFile = NULL;
  }
}

int parseKmlFile(char *fileName, struct image_corners *image, unsigned char *drawing)
{
  double latitude, 
         longitude;
  double xx,       
         yy;
  int    restart = 1;
  int    mode    = 0;

 struct 
  image_corners *imagePtr;
  char          *drawingPtr;

  imagePtr   = image;
  drawingPtr = drawing;

  if (!openKmlFile(fileName)) {
    fprintf(stderr, "Failed to open %s\n", fileName);
    return 0;
  }

  fprintf(stderr, "Debug: parsing Header\n");
  parseKmlHeader();


  fprintf(stderr, "Debug: parsing Track Coordinates\n");
  while (nextKmlCoordinate(&mode, &latitude, &longitude, &restart)) {

    if (CalcDoublePlotPoint(&xx, &yy, longitude, latitude, imagePtr)) {
      switch (mode) {
        case POINT_COORD :
             PlotMarker(xx, yy, imagePtr, drawingPtr, "");
	     restart = 0;
	     break;

        case LINE_COORD  :
	     if (restart) {
	       RestartLine();
	       PlotPoint(xx, yy, imagePtr, drawingPtr, "");
	       restart = 0;
             }
             else {
	       PlotLine(xx, yy, imagePtr, drawingPtr, "");
             }
	     break;

        case POLY_COORD :
             if (restart) {
	       RestartLine();
	       PlotPoint(xx, yy, imagePtr, drawingPtr, "");
	       restart = 0;
             }
             else {
	       PlotLine(xx, yy, imagePtr, drawingPtr, "");
             }	     
	     break;
      }
    }
  }
}
