/* *************************************************************** *
 *  DrawTrails : draws trail data from GPS way-pts.
 *
 * *************************************************************** */

/* Unix includes
 */
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

/* Application includes
 */
#include "drawmap.h"
#include "drawTrails.h"
#include "dem.h"
#include "font_5x8.h"
#include "font_6x10.h"

static double last_x;
static double last_y;
static int    haveLast  =  0;
static unsigned char  markColor = BLACK;

TrailPlotFunc  TrailPlot[PLOT_TYPES] = { PlotPoint, PlotLine, PlotJunction, 
					 PlotMarker, PlotLabel, PlotTick };

int    SetMapUnit(char *s) {
   /* The raw magellen output has the form
    * 
    * NAME  LATITUDE  LONGITUDE  ALTITUDE Description         Icon LMK050
    * 3902.534N 07922.964W 0001182M start
    *
    * Data from the Honcharik's spread sheet has the form
    * 38.986877       -79.3297342     560 Wildlife Tr-GPS'd
    *
    * Garmin_Text data
    *# Header        Position        Time    Altitude        Depth   Length  Time    Speed   Course
    *Trackpoint      N39 02.262 W79 22.880   07/04/2007 12:28:54     1225 m  0.0 m   0 m     0:00:01 0 kph   0° true
    */
  
   char copy[80];          /* Recall strtok modifies itsown input string */
   char *tokPtr; char *decPtr;

   strncpy(copy, s, 79);

   /* This first field is either a waypoint name or the latitude
    */
   if ((tokPtr = strtok(copy, "\t ")) == NULL) 					 
     return NO_MAP_UNIT;

   if ((strcmp(tokPtr, "WayPoint") == 0) || (strcmp(tokPtr, "Trackpoint") == 0)) 
     return GARMIN_TEXT;

   /* The second field is latitude (format xx.xxxx), longitude(dddmm.mmm), [NS]dd mm.mmm
    */
   if ((tokPtr = strtok(NULL, "\t ")) == NULL) 	
     return NO_MAP_UNIT;

   if ((decPtr = strchr(tokPtr, '.')) == NULL) 
     return NO_MAP_UNIT;

   if ( (unsigned int)(decPtr - tokPtr) == 4) 
	return DEGREE_MINUTE; 
   
   if ( (unsigned int)(decPtr - tokPtr) == 3)
	return DECIMAL_DEGREE;

   return NO_MAP_UNIT;
}

void SetPlotColor(char *s)
{ 
  static char *colorList[12] = { "Red",      "Green",      "BrightBlue",   "Black", 
				 "Gray",     "Orange",     "BrightWhite",  "Yellow", 
				 "Purple",   "TrailWhite", "TrailBlue",    "Cyan" };
  int clr;
  
  for (clr=0; clr < 12; clr++) { 
    if (strcasecmp(s, colorList[clr]) == 0) {
	 markColor = C_INDEX_15 + clr; break; 
    } 
  }   
}

double MagellonConv(char *s)
 { 
    int  last    = strlen(s) - 1; 
    char *minPtr = strchr(s, '.') - 2; 
    char *ptr    = s;  
    float deg    = 0; 
    float min;

    while (ptr < minPtr) { 
       deg = deg * 10 + (*ptr - '0'); ptr++; 
    } 

    min = atof(minPtr);

    deg = deg + min / 60.0;

    if ((s[last] == 'W') || (s[last] == 'S')) deg *= -1;

    return deg; }

int parseGpsUtilLine(char *line, double *latitude, double *longitude, char **icon) 
{ 
  char *tok_ptr;

    /* Waypoint name
     */
	if ((tok_ptr = strtok(line, " ")) == NULL) return 0;

    /* Latitude string
     */
	 tok_ptr = strtok(NULL, " "); *latitude = MagellonConv(tok_ptr);

    /* Longitude string 
     */
	 tok_ptr = strtok(NULL, " "); *longitude = MagellonConv(tok_ptr);

    /* Elevation
     */
	 tok_ptr = strtok(NULL, " ");

    /* Comment field or icon
     */
	 *icon = strtok(NULL,"\n");
     
     return 1; }

int parseDecDegreeLine(char *line, double *latitude, double *longitude, char **icon)
{ 
    char *tok_ptr;

    /* Latitude string
     */
       if ((tok_ptr = strtok(line, " \t")) == NULL) 
	  return 0; 
       else 
 	 *latitude = atof(tok_ptr);

    /* Longitude string 
     */
      tok_ptr = strtok(NULL, " \t"); *longitude = atof(tok_ptr);

    *icon = strtok(NULL, "\t");

    return 1;
 }

int parseGarminTextLine(char *line, double *latitude, double *longitude, char **icon)
{
  static char copy[256];
  char  *tokPtr;
  char   latDD[4],
         lonDD[4];
  int    sgn,
         dd;
  double latMM,
         lonMM;
  
  /* make copy to leave line intact */
  strncpy(copy, line, 255);

  /* first field should be "Trackpoint" */
  if ((tokPtr = strtok(copy, "\t")) == NULL) return 0;

  if (strcmp(tokPtr, "Waypoint") == 0)  {  	
    if ((tokPtr = strtok(NULL, "\t")) == NULL) return 0; /* WayPoint name */
    *icon = tokPtr;
    if ((tokPtr = strtok(NULL, "\t")) == NULL) return 0; /* Description unused     */
    if ((tokPtr = strtok(NULL, "\t")) == NULL) return 0; /* Type = "User Waypoint" */
  }
  else {
    *icon = NULL;
  }

  /* next field is latitude as N39 02.262 W79 22.880 */
  if (((tokPtr = strtok(NULL, "\t")) == NULL) || (sscanf(tokPtr, "%s %lf %s %lf", latDD, &latMM, lonDD, &lonMM) != 4))
    return 0;
 
  if (latDD[0] == 'S')
     sgn = -1;
  else if (*tokPtr == 'N')
     sgn = +1;
  else 
    return 0;

  dd = atoi(&latDD[1]);

  *latitude = sgn * ( (double)dd + latMM/60.0);

  if (lonDD[0] == 'W')
    sgn = -1;
  else if (lonDD[0] == 'E')
    sgn = +1;
  else
    return 0;

  dd = atoi(&lonDD[1]);

  *longitude = sgn * ( (double)dd + lonMM/60.0);

  return 1;
}

int FindMarkType(char *s) 
{ 
  if ((s == NULL) || (*s == 'a'))          /* default Magellon icon or nothing */ 
	return LINE;

  if (strncasecmp(s, "start",5) == 0 )     /* start in comments     */ 
	return POINT;

  if (strncasecmp(s, "junction", 8) == 0)  /* junction in comments  */
	return JUNCTION;

  if (strncasecmp(s, "label", 5) == 0)     /* Label in comment      */
	return LABEL;

  if (*s == '+')                           /* marker                */ 
	return MARK;

  if (strncasecmp(s, "tick", 4) == 0)      /* Tick mark V)ert or H)orz */
	return TICK; 

    return POINT; 
}

void RestartLine()
{
  haveLast = 0;
}

void PlotPoint(double x, double y, struct image_corners *imageCorner, unsigned char *bufPtr, char *msg)
 {  
    int i, j;
    long ix, iy;

    /* verify that we are on the map
     */
    if ((x < 0) || (x  > (imageCorner->x - 1)) || ( y < 0) || (y > imageCorner->y-1)) {
       return; 
    }
   
    ix =  lround(x);
    iy =  lround(y);
    for (i=0; i < 2; i++) 
	for (j = 0; j < 2; j++) 
	*(bufPtr + (iy +j + TOP_BORDER) * x_prime + ix +i + LEFT_BORDER) = markColor;

    if ( (msg != NULL) && (strlen(msg) > 2)) {
       long          font_width,
                     font_height;
       unsigned char *font;

    /* Select a font size, based on the image size.
     */
     if ((imageCorner->x >= 1000) && (imageCorner->y >= 1000))  {
        font_width = 6;
        font_height = 10;
        font = &font_6x10[0][0];
     }
     else {
        font_width = 5;
        font_height = 8;
        font = &font_5x8[0][0];
     }

     ix += LEFT_BORDER;
     iy += TOP_BORDER;

     add_text(imageCorner, bufPtr, msg, strlen(msg), ix, iy, font, font_width, font_height, BLACK, WHITE);
    }

    last_x   = x; 
    last_y   = y;
    haveLast = 1;
 }

/* ************************************************************
 * OrderPoints : puts coordinate pair in order based on first
 *               coordinate of pair. Note x/y can be swapped to
 *               order on y
 */
void OrderPoints(double *x1, double *y1, double *x0, double *y0)
{ 
  if (*x1 < *x0) {
    double temp;

    temp = *x0;
    *x0  = *x1;
    *x1  = temp;

    temp = *y0;
    *y0  = *y1;
    *y1  = temp;
  }
}

/* *********************************************************************************
 * MaskLine : Limits line seqment to drawable rectangle
 */
int MaskLine(double *x1, double *y1, double *x0, double *y0, struct image_corners *imageCorner)
{
  int    bothFlag = 0;
  double m;
 
  if ((*x0 >= 0) && (*x0 < imageCorner->x) && (*y0 >= 0) && (*y0 < imageCorner->y) &&
      (*x1 >= 0) && (*x1 < imageCorner->x) && (*y1 >= 0) && (*y1 < imageCorner->y)   ) {
    /* both ends are already inside the rectangle
     */
    return 1;
  }

  if (fabs(*x1-*x0) > fabs(*y1 - *y0)) {               /* X is the dominant motion            */
    OrderPoints(x1, y1, x0, y0);                       /* Put in left-to-right order          */
    if ((*x1 < 0) || (*x0 > imageCorner->x - 1)) {     /* does not intersect area             */
      return 0;
    }

    m = (*y1 - *y0)/(*x1 - *x0);
    
    if (*x0 < 0) {                                     /* First check X range                 */
      *y0 -= *x0*m;                                    /* move y0 up                          */
      *x0  = 0;
    }
    if (*x1 > imageCorner->x - 1) {
      *y1 -= (*x1 - imageCorner->x + 1)*m;             /* move y1 down                        */
      *x1  = imageCorner->x - 1;
    }

    if ( m > 0.00001) {                                /* Now check y range based on order    */
      if ((*y1 < 0) || (*y0 > imageCorner->y-1)) {     /* Completely above or below rectangle */
	return 0;
      }

      if (*y0 < 0) {
        *x0 -= *y0/m;                                  /* move x0 to right                    */
	*y0  = 0;
      }

      if (*y1 > imageCorner->y-1) {
	*x1 -= (*y1 -imageCorner->y +1)/m;             /* move x1 to left                     */
	*y1  = imageCorner->y - 1;
      }
      
    }
    else if ( m < -0.00001) {                          /* Handle m<0                          */
      if ((*y0 < 0) || (*y1 > imageCorner->y-1)) {     /* Completely above or below rectangle */
	return 0;
      }

      if (*y1 < 0) {
        *x1 -= *y1/m;
	*y1 = 0;	
      }

      if (*y0 > imageCorner->y-1) {
	*x0 -= (*y0 -imageCorner->y +1)/m;
	*y0  = imageCorner->y - 1;
      }
    }
    else {                                             /* Handle unlikely m = 0               */
      if ((*y0 < 1) || (*y0 > imageCorner->y-2)) {     /* Flat line above or below rectange   */
	return 0;
      }
    }
  }
  else {                                               /* Delta y is dominant                 */
    OrderPoints(y1, x1, y0, x0);                       /* Put in top-to-bottom order          */
    if ((*y1 < 0) || (*y0 > imageCorner->y - 1)) {     /* does not intersect area             */
      return 0;
    }

    m = (*x1 - *x0)/(*y1 - *y0);
    
    if (*y0 < 0) {                                     /* First check Y range                 */
      *x0 -= *y0*m;                                    /* Move lower point up to y=0          */
      *y0  = 0;
    }
    if (*y1 > imageCorner->y - 1) {
      *x1 -= (*y1 - imageCorner->y + 1)*m;             /* Move uuper point to edge            */
      *y1  = imageCorner->y - 1;
    }

    if ( m > 0.00001) {                                /* Now check x range based on order    */
      if ((*x1 < 0) || (*x0 > imageCorner->x-1)) {     /* Completely above or below rectangle */
	return 0;
      }

      if (*x0 < 0) {
        *y0 -= *x0/m;
	*x0  = 0;
      }

      if (*x1 > imageCorner->x-1) {
	*y1 -= (*x1 -imageCorner->x +1)/m;
	*x1  = imageCorner->x - 1;
      }
      
    }
    else if ( m < -0.00001) {                          /* Handle m<0                          */
      if ((*x0 < 0) || (*x1 > imageCorner->x-1)) {     /* Completely above or below rectangle */
	return 0;
      }

      if (*x1 < 0) {
        *y1 -= *x1/m;
	*x1  = 0;	
      }

      if (*x0 > imageCorner->x-1) {
	*y0 -= (*x0 -imageCorner->x +1)/m;
	*x0  = imageCorner->x - 1;
      }
    }
    else {                                             /* Handle unlikely m = 0               */
      if ((*y0 < 0) || (*y0 > imageCorner->y-1)) {     /* Flat line above or below rectange   */
	return 0;
      }
    }
  }
                                                       /* Passed tests, have data to plot     */
  return 1;                                           
}

void PlotLine(double x, double y, struct image_corners *imageCorner, unsigned char *bufPtr, char *msg)
{
  int     i,  iCnt,
          ix, iy, t;
  double  rx,  /* plot points as doubles              */
	  ry, 
	  dx,  /* x component of the unit vector      */
          dy,  /* y component of the unit vector      */
          dtx,
          tx,  /* x component of the thick line       */
          dty,
          ty;  /* y component of the thich line       */

  double  x0,y0,
          x1,y1;


  if (!haveLast) {
    PlotPoint(x, y, imageCorner, bufPtr, msg);
    
    last_x   = x;                                   /* Force saving last point for MaskLine */
    last_y   = y;
    haveLast = 1;
    return;
  }

  x1 = x;                                           /* Copy line ends to working points     */
  y1 = y;
  x0 = last_x;
  y0 = last_y;

  if (MaskLine(&x1, &y1, &x0, &y0, imageCorner)) {  /* Clip line ends to drawable rectangle */

    dx = (x1 - x0);
    dy = (y1 - y0);
      
    if ((fabs(dx)< 1.5) && (fabs(dy)<1.5)) {
      PlotPoint(x1, y1, imageCorner, bufPtr, msg);
      last_x   = x;                                 /* Force saving last point for MaskLine */
      last_y   = y;
      haveLast = 1;
      return;
    }

    if (fabs(dx) > fabs(dy)) {
      iCnt = (int)(2*fabs(dx)+0.5);
      dtx  = 0;
      dty  = 0.50;
      dx   = dx/iCnt;
      dy   = dy/iCnt;
    }
    else {
      iCnt = (int)(2*fabs(dy)+0.5);
      dtx  = 0.50;
      dty  = 0;
      dx   = dx/iCnt;
      dy   = dy/iCnt;
    }
    
    rx    = x0;
    ry    = y0;
    iCnt += 1;
    
    for (i = 0; i < iCnt; i++) {
      if ((rx < 0) || (rx  > (imageCorner->x - 1)) || ( ry < 0) || (ry > imageCorner->y-1))
	continue;
      
      for (t=0,tx=-dtx,ty=-dty; t < 3; t++,tx += dtx,ty += dty) {
	ix = lround(rx+tx);
	iy = lround(ry+ty);
	*(bufPtr + (iy + TOP_BORDER) * x_prime + ix + LEFT_BORDER) = markColor;
      }
      rx += dx;
      ry += dy;
    }
  }  
  last_x   = x;
  last_y   = y;
  haveLast = 1;
}  

void PlotJunction(double x, double y, struct image_corners *imageCorner, unsigned char *bufPtr, char *msg)
  {
    int        i, ix, iy;
    double     rx, ry, 
	       dx,  /* x component of the unit vector      */
               dy,  /* y component of the unit vector      */
               tx,  /* x component of the thick line       */
               ty,  /* y component of the thich line       */
               dr;  /* magnitude of change from last point */

    /* verify that we are on the map
     */
      if ((x < 0) || (x  > (imageCorner->x - 1)) || ( y < 0) || (y > imageCorner->y-1))
        return;  

    /* determine heading from last point
     */
      dx = (last_x - x);
      dy = (last_y - y);
      dr = sqrt(dx*dx + dy*dy);
      dx = dx / dr;
      dy = dy / dr;

      tx = dy;
      ty = -dx;

      for (i = -4; i < 0; i++) {
         rx = i*dx + last_x;
         ry = i*dy + last_y;
         if ((rx < 0) || (rx  > (imageCorner->x - 1)) || ( ry < 0) || (ry > imageCorner->y-1))
            continue;

	 ix = lround(rx);
	 iy = lround(ry);
         *(bufPtr + (iy + TOP_BORDER) * x_prime + ix + LEFT_BORDER) = markColor;
	 rx += tx;
	 ry += ty;

         ix = lround(rx);
         iy = lround(ry);
         *(bufPtr + (iy + TOP_BORDER) * x_prime + ix + LEFT_BORDER) = markColor;
	}

      for (i = 0; i < 4; i++) {
         rx = i*dx + x;
         ry = i*dy + y;
         if ((rx < 0) || (rx  > (imageCorner->x - 1)) || ( ry < 0) || (ry > imageCorner->y-1))
            continue;
          
         ix = lround(rx);
         iy = lround(ry);
          *(bufPtr + (iy + TOP_BORDER) * x_prime + ix + LEFT_BORDER) = markColor;

	 rx += tx;
	 ry += ty;
         ix = lround(rx);
         iy = lround(ry);
          *(bufPtr + (iy + TOP_BORDER) * x_prime + ix + LEFT_BORDER) = markColor;
        }

    /* rotate heading 90 degrees, dr here is just temporary holder
     */

       for (i = -4; i < 5; i++) {
         rx = i*tx + x;
         ry = i*ty + y;
         if ((rx < 0) || (rx  > (imageCorner->x - 1)) || ( ry < 0) || (ry > imageCorner->y-1))
            continue;

         ix = lround(rx);
         iy = lround(ry);
         *(bufPtr + (iy + TOP_BORDER) * x_prime + ix + LEFT_BORDER) = markColor;
	 }
    
      last_x = x;
      last_y = y;
  }

void PlotMarker(double x, double y, struct image_corners *imageCorner, unsigned char *bufPtr, char *msg)
{
  long i, ix, iy;

  /* verify that we are on the map
   */
  if ((x < 0) || (x  > (imageCorner->x - 1)) || ( y < 0) || (y > imageCorner->y-1))
    return;  

  ix = lround(x);
  iy = lround(y); 
  for (i = -2; i <= 2; i++) {
    if (((ix + i) >= 0) && ((ix + i) <= (imageCorner->x - 1)) &&
        ( iy      >= 0) && ( iy      <= (imageCorner->y - 1))    )
      *(bufPtr + (iy + TOP_BORDER    ) * x_prime + ix + LEFT_BORDER + i) = markColor;
    		       
    if (( ix      >= 0) && ( ix      <= (imageCorner->x - 1)) &&
        ((iy + i) >= 0) && ((iy + i) <= (imageCorner->y - 1))    )
      *(bufPtr + (iy + TOP_BORDER + i) * x_prime + ix + LEFT_BORDER    ) = markColor;
  
  }
}

void PlotTick(double x, double y, struct image_corners *imageCorner, unsigned char *bufPtr, char *data)
{
  long i, ix, iy;
  
  /* verify that we are on the map
   */
  if ((x < 0) || (x  > (imageCorner->x - 1)) || ( y < 0) || (y > imageCorner->y-1))
    return;  

  data += strlen("tick ");
  ix = lround(x);
  iy = lround(y); 

  switch (data[0]) {
    case 'v':
    case 'V': for (i = -2; i <= 2; i++) {	       
	        if (((iy + i) >= 0) && ((iy + i) <= (imageCorner->y - 1)))  
	          *(bufPtr + (iy + TOP_BORDER + i) * x_prime + ix + LEFT_BORDER) = markColor;
               }
              break;
    case 'h':
    case 'H': for (i = -2; i <= 2; i++) {
	        if (((ix + i) >= 0) && ((ix + i) <= (imageCorner->x - 1)))
	          *(bufPtr + (iy + TOP_BORDER) * x_prime + ix + LEFT_BORDER + i) = markColor;
               }
              break;
   }
}

void PlotLabel(double x, double y, struct image_corners *imageCorner, unsigned char *bufPtr, char *msg)
{
  long          ix,
                iy,
                font_width,
                font_height;
  unsigned char *font;

  /* verify that we are on the map
   */
  if ((x < 0) || (x  > (imageCorner->x - 1)) || ( y < 0) || (y > imageCorner->y-1))
    return; 

   
  ix = lround(x);
  iy = lround(y); 
  /* Select a font size, based on the image size.
   */
  if ((imageCorner->x >= 1000) && (imageCorner->y >= 1000))  {
    font_width  =  6;
    font_height = 10;
    font = &font_6x10[0][0];
  }
  else  {
    font_width  = 5;
    font_height = 8;
    font = &font_5x8[0][0];
  }

  /* step over label key word to label text
   */
  if (strncmp(msg,"label", 5) == 0) {
    msg += strlen("label ");
  }

  ix += LEFT_BORDER;
  iy += TOP_BORDER;
  
  add_text(imageCorner, bufPtr, msg, strlen(msg), ix, iy, font, font_width, font_height, BLACK, WHITE);
}


/* Convert longitude,latitude to image location as doubles (x,y)
 */
int CalcDoublePlotPoint(double *x, double *y, double longitude, double latitude, struct image_corners *imageCorner)
{
   double del_x = 0;
   double del_y = 0;

   /* Ignore this entry if it is out of the map area.
    *
    *  if ((latitude  < imageCorner->sw_lat ) || (latitude  > imageCorner->ne_lat ) ||
    *	  (longitude < imageCorner->sw_long) || (longitude > imageCorner->ne_long)  )
    *    return 0;
    */

   /* Calc of del_x, del_y
    */
      del_x = (double)imageCorner->x / (imageCorner->ne_long - imageCorner->sw_long);
      del_y = (double)imageCorner->y / (imageCorner->ne_lat  - imageCorner->sw_lat );

   /* Have valid point
    */
     *x =                  (longitude - imageCorner->sw_long) * del_x - 1;
     *y = imageCorner->y - (latitude  - imageCorner->sw_lat ) * del_y - 1;

     return 1;
}

/* PlotScale of miles and kilometers at bottom center of map
 */
int PlotScale(struct datum *datum, struct image_corners *imageCorner, unsigned char *bufPtr)
{
  double xPosUtm, 
         yPosUtm,
         xCtrUtm,
         yCtrUtm;
  double lonCtr = 0.5  * imageCorner->sw_long + 0.5  * imageCorner->ne_long,
         latCtr = 0.95 * imageCorner->sw_lat  + 0.05 * imageCorner->ne_lat, 
         latDel = (imageCorner->ne_lat - imageCorner->sw_lat)/imageCorner->y,
         lonPos,
         latPos;
  int    i,j;
  long   x,y;
  char  *km     = "  km  ";
  char  *mile   = " mile ";

  redfearn(datum, &xCtrUtm, &yCtrUtm, &imageCorner->sw_zone, latCtr, lonCtr, 1);

  SetPlotColor("Black");
  for (i=-3000,j=-3; i<2001; i += 1000,j=(j== -3)?3:-3){
    xPosUtm = xCtrUtm+i;

    redfearn_inverse(datum, xPosUtm,      yCtrUtm , imageCorner->sw_zone, &latPos, &lonPos);
    CalcPlotPoint(&x, &y, lonPos, latCtr+j*latDel, imageCorner);
    PlotPoint(x, y, imageCorner, bufPtr, NULL);

    redfearn_inverse(datum, xPosUtm+1000, yCtrUtm , imageCorner->sw_zone, &latPos, &lonPos);
    CalcPlotPoint(&x, &y, lonPos, latCtr+j*latDel, imageCorner); 
    PlotLine(x-1, y,  imageCorner, bufPtr, NULL);
  }

  PlotLabel(x+10, y,  imageCorner, bufPtr, km);


  latCtr -= 20 * latDel;
  for (i=-4827,j=-3; i<3220; i += 1609,j=(j== -3)?3:-3){
    xPosUtm = xCtrUtm+i;

    redfearn_inverse(datum, xPosUtm,      yCtrUtm , imageCorner->sw_zone, &latPos, &lonPos);
    CalcPlotPoint(&x, &y, lonPos, latCtr+j*latDel, imageCorner);
    PlotPoint(x, y, imageCorner, bufPtr, NULL);

    redfearn_inverse(datum, xPosUtm+1609, yCtrUtm , imageCorner->sw_zone, &latPos, &lonPos);
    CalcPlotPoint(&x, &y, lonPos, latCtr+j*latDel, imageCorner); 
    PlotLine(x-1, y,  imageCorner, bufPtr, NULL);
  }

  PlotLabel(x+10, y, imageCorner, bufPtr, mile);

 
  return 1;

}
/* =================================================================================
 *                               E N D   O F   F I L E          
 * ================================================================================= */
