/* ==============================================================================
 *
 * utmCalcUtils : Mapping from UTM based data grid to Longitude/Latitude based
 *                data grids.  This follows from Fred M. Erickson in drawmap.c
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Here is the specific quotes from drawmap.c ....
 * 
 > Note:  The mapping of DEM data into the image is done by simple linear interpolation
 > from the edges of the DEM data.  Strictly speaking, this is not quite correct for
 > 7.5-minute DEM data, because it treats the boundaries of the DEM data as a perfect rectangle.
 > However, I implemented a much more complicated method that went like this:
 >
 >	Use the (i, j) location in the image to determine an accurate latitude/longitude.
 >      Map the latitude/longitude into UTM coordinates with the redfearn() function.
 >      Use these UTM coordinates, along with the known UTM coordinates of the first
 >          point in the first profile of the DEM, to accurately determine the correct
 >          (k, l) point in the DEM data that corresponds to the specified latitude/longitude.
 >      Use that correct point (and the surrounding points) to produce an elevation
 >          value to stuff into the (i, j) location in the image.
 >
 > Technically speaking, this is about as accurate a job as can be done without implementing
 > some between-point interpolation.  It made a minor difference in the image, but
 > it wasn't clearly better (in my opinion), just slightly different.  I thus reverted
 > back to this, simpler, more-efficient approach.
 >
 * In attempting to map hiking trails onto DEM based conture maps, I found that the 
 * simple approach mentioned above was not accurate enough. 
 * 
 * ===================================================================================== */

/* Standard Includes =================================================================
 */
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <math.h>


/* Application Includes ==============================================================
 */
#include "drawmap.h"
#include "dem.h"
#include "raster.h"
#include "utmCalcUtils.h"
 
/* Static Variables local to this module =============================================
 */
    static double x_gp_first,     y_gp_first,
                  x_gp_slope,     y_gp_slope,
                  x_gp_offset,    y_gp_offset,
                  x_gp_last,      y_gp_last;

    static double del_lat,        del_long;


/* Functions =========================================================================
 */

/* ---------------------------------------------------------------------------------
 * NewDemData : resets local variables base on the dem date
 */
void NewDemData( struct dem_record_type_a *dem)
  {
    x_gp_first  = dem->x_gp_first;
    y_gp_first  = dem->y_gp_first;

    x_gp_last   = dem->x_gp_first +  dem->x_res * dem->cols;
    y_gp_last   = dem->y_gp_first -  dem->y_res * dem->rows;
    
    x_gp_slope  = 1.0 / dem->x_res;
    y_gp_slope  = 1.0 / dem->y_res;

    x_gp_offset = dem->x_gp_first * x_gp_slope;
    y_gp_offset = dem->y_gp_first * y_gp_slope;
  }

void NewImageGrid( struct image_corners *imageCorner)
  {
     del_lat  = (imageCorner->ne_lat  - imageCorner->sw_lat ) / (double)imageCorner->y;
     del_long = (imageCorner->ne_long - imageCorner->sw_long) / (double)imageCorner->x;
  }

/* --------------------------------------------------------------------------------
 * CalcLatitudePoint : calc latitude from the y coordinate of the image point.
 */
int CalcLatitudePoint(double *latitude, long y, struct image_corners *imageCorner)
  {
 
    /* preform range check
     */
      if ((y < 0) || (y > imageCorner->y))
        return 0;

    /* calc long and latitude
     */
      *latitude  = (imageCorner->y - y + 1) * del_lat  + imageCorner->sw_lat;

      return 1;
  }

/* --------------------------------------------------------------------------------
 * CalcLongitudePoint : calc latitude from the x coordinate of the image point.
 */
int CalcLongitudePoint(double *longitude, long x,  struct image_corners *imageCorner)
  {
    /* preform range check
     */
      if ((x < 0) || (x > imageCorner->x))
        return 0;

    /* calc long and latitude
     */
      *longitude = (x + 1) * del_long + imageCorner->sw_long;

      return 1;
  }

/* ---------------------------------------------------------------------------------
 * UTMtoIJ : Convert UTM coordinate to data grid points
 */
int UTMtoIJ(long *ix, long *iy, double xUtm, double yUtm)
  {
   if ( (xUtm < x_gp_first) || (yUtm > y_gp_first) || (xUtm > x_gp_last) || (yUtm < y_gp_last) )
     return 0;
   else
      {
	*ix = (int) ( 0.49 + xUtm * x_gp_slope - x_gp_offset      );
        *iy = (int) ( 0.49 + y_gp_offset       - yUtm * y_gp_slope);

        return 1;
      } 
  }

/* --------------------------------------------------------------------------------
 * ImageToDataGrid : finds the closest UTM elevation data point
 */
int ImageToDataGrid( long *xGrid, long *yGrid, double longitude, double latitude,
                     struct datum *datum)
  {    
    double xUtm,  yUtm;
    long   zone;

    if (redfearn(datum, &xUtm, &yUtm, &zone, latitude, longitude, 0) != 0)
      return 0;

    if (!UTMtoIJ(xGrid, yGrid, xUtm, yUtm))
      return 0;

    return 1;
  }

/* ===================================================================================
 *                                E N D   O F   F I L E
 * ===================================================================================
 */


